@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5.2/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/Utils/ConstructBoundingBox.h $
-// $Id: ConstructBoundingBox.h 1d3815f 2020-10-02T17:29:03+02:00 Ahmed Essam
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.2/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/Utils/ConstructBoundingBox.h $
+// $Id: ConstructBoundingBox.h 6f103e4de15 2022-09-11T00:30:15+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s): Ahmed Essam <theartful.ae@gmail.com>
@@ -17,15 +17,23 @@
 // bounding box utility for arrangements
 // doesn't have to be exact, only good enough for rendering
 template <typename Traits_>
-class ConstructBoundingBox
-{
+class ConstructBoundingBox {
 public:
   using Traits = Traits_;
   using X_monotone_curve_2 = typename Traits::X_monotone_curve_2;
   using Point_2 = typename Traits::Point_2;
 
+  // Construct from traits.
+  ConstructBoundingBox(const Traits& traits) : m_traits(traits) {}
+
+  //
   CGAL::Bbox_2 operator()(const X_monotone_curve_2& curve);
+
+  //
   CGAL::Bbox_2 operator()(const Point_2& point);
+
+private:
+  const Traits& m_traits;
 };
 
 #endif
