@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5.2/Linear_cell_complex/demo/Linear_cell_complex/Viewer.h $
-// $Id: Viewer.h fb6f703 2021-05-04T14:07:49+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.2/Linear_cell_complex/demo/Linear_cell_complex/Viewer.h $
+// $Id: Viewer.h 999a813b35e 2022-05-05T13:34:19+02:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -22,69 +22,69 @@ struct MyDrawingFunctorLCC
   /// @return true iff the volume containing dh is drawn.
   template<typename LCC>
   bool draw_volume(const LCC& alcc,
-                   typename LCC::Dart_const_handle dh) const
+                   typename LCC::Dart_const_descriptor dh) const
   { return alcc.template info<3>(dh).is_visible(); }
   /// @return true iff the face containing dh is drawn.
   template<typename LCC>
   bool draw_face(const LCC&,
-                 typename LCC::Dart_const_handle) const
+                 typename LCC::Dart_const_descriptor) const
   { return true; }
   /// @return true iff the edge containing dh is drawn.
   template<typename LCC>
   bool draw_edge(const LCC&,
-                 typename LCC::Dart_const_handle) const
+                 typename LCC::Dart_const_descriptor) const
   { return true; }
   /// @return true iff the vertex containing dh is drawn.
   template<typename LCC>
   bool draw_vertex(const LCC&,
-                   typename LCC::Dart_const_handle) const
+                   typename LCC::Dart_const_descriptor) const
   { return true; }
 
   /// @return true iff the volume containing dh is drawn in wireframe.
   template<typename LCC>
   bool volume_wireframe(const LCC& alcc,
-                        typename LCC::Dart_const_handle dh) const
+                        typename LCC::Dart_const_descriptor dh) const
   { return !(alcc.template info<3>(dh).is_filled()); }
   /// @return true iff the face containing dh is drawn in wireframe.
   template<typename LCC>
   bool face_wireframe(const LCC&,
-                        typename LCC::Dart_const_handle) const
+                        typename LCC::Dart_const_descriptor) const
   { return false; }
 
   /// @return true iff the volume containing dh is colored.
   template<typename LCC>
   bool colored_volume(const LCC&,
-                      typename LCC::Dart_const_handle) const
+                      typename LCC::Dart_const_descriptor) const
   { return true; }
   /// @return true iff the face containing dh is colored.
   ///  if we have also colored_volume(alcc, dh), the volume color is
   ///  ignored and only the face color is considered.
   template<typename LCC>
   bool colored_face(const LCC&,
-                    typename LCC::Dart_const_handle) const
+                    typename LCC::Dart_const_descriptor) const
   { return false; }
   /// @return true iff the edge containing dh is colored.
   template<typename LCC>
   bool colored_edge(const LCC&,
-                    typename LCC::Dart_const_handle) const
+                    typename LCC::Dart_const_descriptor) const
   { return false; }
   /// @return true iff the vertex containing dh is colored.
   template<typename LCC>
   bool colored_vertex(const LCC&,
-                      typename LCC::Dart_const_handle) const
+                      typename LCC::Dart_const_descriptor) const
   { return false; }
 
   /// @return the color of the volume containing dh
   ///  used only if colored_volume(alcc, dh) and !colored_face(alcc, dh)
   template<typename LCC>
   CGAL::IO::Color volume_color(const LCC& alcc,
-                           typename LCC::Dart_const_handle dh) const
+                           typename LCC::Dart_const_descriptor dh) const
   { return alcc.template info<3>(dh).color(); }
   /// @return the color of the face containing dh
   ///  used only if colored_face(alcc, dh)
   template<typename LCC>
   CGAL::IO::Color face_color(const LCC& alcc,
-                         typename LCC::Dart_const_handle dh) const
+                         typename LCC::Dart_const_descriptor dh) const
   {
     CGAL::Random random((unsigned int)(alcc.darts().index(dh)));
     return get_random_color(random);
@@ -93,13 +93,13 @@ struct MyDrawingFunctorLCC
   ///  used only if colored_edge(alcc, dh)
   template<typename LCC>
   CGAL::IO::Color edge_color(const LCC&,
-                         typename LCC::Dart_const_handle) const
+                         typename LCC::Dart_const_descriptor) const
   { return CGAL::IO::Color(0, 0, 0); }
   /// @return the color of the vertex containing dh
   ///  used only if colored_vertex(alcc, dh)
   template<typename LCC>
   CGAL::IO::Color vertex_color(const LCC&,
-                           typename LCC::Dart_const_handle) const
+                           typename LCC::Dart_const_descriptor) const
   { return CGAL::IO::Color(0, 0, 0); }
 };
 
