@@ -14,14 +14,11 @@
 
 #include <CGAL/iterator.h>
 #include <CGAL/Polygon_mesh_processing/remesh.h>
-#include <CGAL/boost/graph/graph_traits_Polyhedron_3.h>
-#include <CGAL/boost/graph/properties_Polyhedron_3.h>
 #include <CGAL/Polygon_mesh_processing/triangulate_faces.h>
 #include <CGAL/utility.h>
+#include <CGAL/property_map.h>
 
 #include <boost/graph/graph_traits.hpp>
-#include <boost/unordered_set.hpp>
-#include <CGAL/property_map.h>
 
 #include <QElapsedTimer>
 #include <QAction>
@@ -37,6 +34,7 @@
 #include <queue>
 #include <sstream>
 #include <cmath>
+#include <unordered_set>
 
 #ifdef CGAL_LINKED_WITH_TBB
 #include "tbb/parallel_for.h"
@@ -51,6 +49,7 @@ typedef Scene_surface_mesh_item Scene_facegraph_item;
 typedef Scene_facegraph_item::Face_graph FaceGraph;
 typedef boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
 
+
 // give a halfedge and a target edge length, put in `out` points
 // which the edge equally spaced such that splitting the edge
 // using the sequence of points make the edges shorter than
@@ -98,7 +97,7 @@ split_identical_edges(
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::halfedge_descriptor halfedge_descriptor;
 
-  BOOST_FOREACH(const Point_3& p, points)
+  for(const Point_3& p : points)
   {
     // split the edge
     halfedge_descriptor new_hd=CGAL::Euler::split_edge(hd,tm);
@@ -144,7 +143,7 @@ void split_long_duplicated_edge(const HedgeRange& hedge_range,
   CGAL_assertion_code(Point_3 tgt = get(pmap, target(hd, *p.second));)
 
   // split the edges and collect faces to triangulate
-  BOOST_FOREACH(const Pair& h_and_p, hedge_range)
+  for(const Pair& h_and_p : hedge_range)
   {
     halfedge_descriptor hc=h_and_p.first;
     TriangleMesh* polyc = h_and_p.second;
@@ -170,15 +169,37 @@ class Polyhedron_demo_isotropic_remeshing_plugin :
 {
   Q_OBJECT
   Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
-  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0" FILE "isotropic_remeshing_plugin.json")
 
   typedef boost::graph_traits<FaceGraph>::edge_descriptor edge_descriptor;
   typedef boost::graph_traits<FaceGraph>::halfedge_descriptor halfedge_descriptor;
   typedef boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
 
-  typedef boost::unordered_set<edge_descriptor>    Edge_set;
+  typedef std::unordered_set<edge_descriptor>    Edge_set;
   typedef Scene_polyhedron_selection_item::Is_constrained_map<Edge_set> Edge_constrained_pmap;
 
+  struct Selection_updater_visitor
+    : public CGAL::Polygon_mesh_processing::Hole_filling::Default_visitor
+  {
+    typedef typename Scene_polyhedron_selection_item::Selection_set_facet Container;
+    Container& faces;
+
+    Selection_updater_visitor(Container& container)
+      : faces(container)
+    {}
+
+    void before_subface_creations(face_descriptor fd)
+    {
+      Container::iterator it = faces.find(fd);
+      faces.erase(it);
+    }
+    void after_subface_created(face_descriptor fd)
+    {
+      faces.insert(fd);
+    }
+    void after_subface_creations(){}
+  };
+
 public:
   void init(QMainWindow* mainWindow, Scene_interface* scene_interface, Messages_interface*)
   {
@@ -205,13 +226,16 @@ public:
     || qobject_cast<Scene_polyhedron_selection_item*>(scene->item(scene->mainSelectionIndex()));
     }
 
+    bool ok(true), found_poly(false);
+
     Q_FOREACH(int index, scene->selectionIndices())
     {
-      //if one polyhedron is found in the selection, it's fine
-      if (qobject_cast<Scene_facegraph_item*>(scene->item(index)))
-        return true;
+      if (!qobject_cast<Scene_facegraph_item*>(scene->item(index)))
+        ok = false;
+      else
+        found_poly=true;
     }
-    return false;
+    return ok && found_poly;
   }
 
   typedef boost::property_map<FaceGraph, CGAL::face_patch_id_t<int> >::type Patch_id_pmap;
@@ -229,10 +253,10 @@ public:
       CGAL::vertex_point_t>::type PointPMap;
     MapType duplicated_edges;
 
-    BOOST_FOREACH(Scene_facegraph_item* poly_item, selection){
+    for(Scene_facegraph_item* poly_item : selection){
       FaceGraph& pmesh = *poly_item->polyhedron();
       PointPMap pmap = get(boost::vertex_point, pmesh);
-      BOOST_FOREACH(edge_descriptor ed, edges(pmesh)){
+      for(edge_descriptor ed : edges(pmesh)){
         halfedge_descriptor hd = halfedge(ed,pmesh);
         Point_3 p = get(pmap, source(hd,pmesh)), q = get(pmap, target(hd,pmesh));
         Segment_3 s = CGAL::make_sorted_pair(p,q);
@@ -245,11 +269,11 @@ public:
     // consistently split duplicate edges and triangulate incident faces
     typedef std::pair<face_descriptor, FaceGraph*> Face_and_poly;
     std::set< Face_and_poly > faces_to_triangulate;
-    BOOST_FOREACH(const MapType::value_type& p, duplicated_edges)
+    for(const MapType::value_type& p : duplicated_edges)
       if (p.second.size()>1){
         //collect faces to retriangulate
         typedef std::pair<halfedge_descriptor, FaceGraph*> Pair_type;
-        BOOST_FOREACH(const Pair_type& h_and_p, p.second)
+        for(const Pair_type& h_and_p : p.second)
         {
           halfedge_descriptor hc=h_and_p.first;
           FaceGraph* polyc = h_and_p.second;
@@ -265,7 +289,7 @@ public:
       }
     // now retriangulate
     namespace PMP=CGAL::Polygon_mesh_processing;
-    BOOST_FOREACH(Face_and_poly f_and_p, faces_to_triangulate)
+    for(Face_and_poly f_and_p : faces_to_triangulate)
       PMP::triangulate_face(f_and_p.first, *f_and_p.second);
   }
 
@@ -274,14 +298,14 @@ public:
                       double target_length)
   {
     std::vector<edge_descriptor> p_edges;
-    BOOST_FOREACH(edge_descriptor e, edges(pmesh))
+    for(edge_descriptor e : edges(pmesh))
     {
       if(get(selection_item->constrained_edges_pmap(), e))
         p_edges.push_back(e);
     }
-    BOOST_FOREACH(face_descriptor f, selection_item->selected_facets)
+    for(face_descriptor f : selection_item->selected_facets)
     {
-      BOOST_FOREACH(halfedge_descriptor he, halfedges_around_face(halfedge(f, pmesh), pmesh))
+      for(halfedge_descriptor he : halfedges_around_face(halfedge(f, pmesh), pmesh))
       {
         if (selection_item->selected_facets.find(face(opposite(he, pmesh), pmesh))
             == selection_item->selected_facets.end())
@@ -293,7 +317,7 @@ public:
             p_edges
             , target_length
             , *selection_item->polyhedron()
-            , PMP::parameters::geom_traits(EPICK())
+            , CGAL::parameters::geom_traits(EPICK())
             .edge_is_constrained_map(selection_item->constrained_edges_pmap()));
     else
       std::cout << "No selected or boundary edges to be split" << std::endl;
@@ -317,10 +341,14 @@ public Q_SLOTS:
 
     if (poly_item || selection_item)
     {
+      if(selection_item && selection_item->selected_edges.empty() && selection_item->selected_facets.empty())
+      {
+        QMessageBox::warning(mw, "Empty Edges", "There are no selected edges. Aborting.");
+        return;
+      }
       // Create dialog box
       QDialog dialog(mw);
-      Ui::Isotropic_remeshing_dialog ui
-        = remeshing_dialog(&dialog, poly_item, selection_item);
+      initialize_remeshing_dialog(&dialog, poly_item, selection_item);
 
       // Get values
       int i = dialog.exec();
@@ -329,6 +357,7 @@ public Q_SLOTS:
         std::cout << "Remeshing aborted" << std::endl;
         return;
       }
+
       bool edges_only = ui.splitEdgesOnly_checkbox->isChecked();
       bool preserve_duplicates = ui.preserveDuplicates_checkbox->isChecked();
       double target_length = ui.edgeLength_dspinbox->value();
@@ -346,15 +375,14 @@ public Q_SLOTS:
       typedef boost::graph_traits<FaceGraph>::edge_descriptor edge_descriptor;
       typedef boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
 
-      FaceGraph& pmesh = (poly_item != NULL)
+      FaceGraph& pmesh = (poly_item != nullptr)
         ? *poly_item->polyhedron()
         : *selection_item->polyhedron();
 
-
      Patch_id_pmap fpmap = get(CGAL::face_patch_id_t<int>(), pmesh);
      bool fpmap_valid = false;
      {
-       BOOST_FOREACH(face_descriptor f, faces(pmesh))
+       for(face_descriptor f : faces(pmesh))
        {
          if (get(fpmap, f) != 1)
          {
@@ -377,14 +405,14 @@ public Q_SLOTS:
                  *selection_item->polyhedron(),
                  selection_item->constrained_edges_pmap(),
                  get(CGAL::vertex_point, *selection_item->polyhedron()),
-                 CGAL::Static_property_map<face_descriptor, std::size_t>(1),
+                 CGAL::Constant_property_map<face_descriptor, std::size_t>(1),
                  4. / 3. * target_length))
             {
               QApplication::restoreOverrideCursor();
               //If facets are selected, splitting edges will add facets that won't be selected, and it will mess up the rest.
-              //If there is only edges, it will work fine because new edges are dealt with in the code, so we can directly 
+              //If there is only edges, it will work fine because new edges are dealt with in the code, so we can directly
               //split and continue.
-              // Possibility todo: check if the barycenter of a new face is inside an old selected face to 
+              // Possibility todo: check if the barycenter of a new face is inside an old selected face to
               //select it again.
               if(!selection_item->selected_facets.empty())
               {
@@ -397,25 +425,43 @@ public Q_SLOTS:
               else if(QMessageBox::question(mw, tr("Error"),
                                             tr("Isotropic remeshing : protect_constraints cannot be set to"
                                                " true with constraints larger than 4/3 * target_edge_length."
-                                               " Do you wish to split the constrained edges ?")) != 
+                                               " Do you wish to split the constrained edges ?")) !=
                       QMessageBox::Yes)
               {
                 return;
               }
               else
               {
-                QApplication::setOverrideCursor(Qt::WaitCursor);
                 do_split_edges(selection_item, pmesh, target_length);
               }
             }
 
             if (selection_item->selected_facets.empty() && !selection_item->isEmpty())
             {
+              if (!CGAL::is_triangle_mesh(pmesh))
+              {
+                QApplication::restoreOverrideCursor();
+                if (QMessageBox::Ok ==
+                    QMessageBox::question(mw, tr("Error - Triangulate Faces?"),
+                      tr("The input mesh is not a triangulated surface mesh.\n"
+                         "Do you wish to triangulate faces first, or cancel remeshing ?"),
+                      (QMessageBox::Ok | QMessageBox::Cancel),
+                      QMessageBox::Ok))
+                {
+                  QApplication::setOverrideCursor(Qt::WaitCursor);
+                  CGAL::Polygon_mesh_processing::triangulate_faces(pmesh);
+                }
+                else
+                {
+                  return;
+                }
+              }
+
               if (fpmap_valid)
                 CGAL::Polygon_mesh_processing::isotropic_remeshing(faces(*selection_item->polyhedron())
                    , target_length
                    , *selection_item->polyhedron()
-                   , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter)
+                   , CGAL::parameters::number_of_iterations(nb_iter)
                    .protect_constraints(protect)
                    .edge_is_constrained_map(selection_item->constrained_edges_pmap())
                    .relax_constraints(smooth_features)
@@ -426,7 +472,7 @@ public Q_SLOTS:
                 CGAL::Polygon_mesh_processing::isotropic_remeshing(faces(*selection_item->polyhedron())
                    , target_length
                    , *selection_item->polyhedron()
-                   , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter)
+                   , CGAL::parameters::number_of_iterations(nb_iter)
                    .protect_constraints(protect)
                    .edge_is_constrained_map(selection_item->constrained_edges_pmap())
                    .relax_constraints(smooth_features)
@@ -436,11 +482,36 @@ public Q_SLOTS:
             }
             else //selected_facets not empty
             {
+              for (auto f : selection_item->selected_facets)
+              {
+                if (!CGAL::is_triangle(halfedge(f, pmesh), pmesh))
+                {
+                  QApplication::restoreOverrideCursor();
+                  if(QMessageBox::Ok ==
+                     QMessageBox::question(mw, tr("Error - Triangulate Faces?"),
+                       tr("The input faces selected for remeshing are not all triangle faces.\n"
+                          "Do you wish to triangulate faces first, or cancel remeshing ?"),
+                       (QMessageBox::Ok | QMessageBox::Cancel),
+                       QMessageBox::Ok))
+                  {
+                    Selection_updater_visitor visitor(selection_item->selected_facets);
+                    CGAL::Polygon_mesh_processing::triangulate_faces(selection_item->selected_facets,
+                      pmesh,
+                      CGAL::parameters::visitor(visitor));
+                    break;
+                  }
+                  else
+                  {
+                    return;
+                  }
+                }
+              }
+
               if (fpmap_valid)
                 CGAL::Polygon_mesh_processing::isotropic_remeshing(selection_item->selected_facets
                   , target_length
                   , *selection_item->polyhedron()
-                  , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter)
+                  , CGAL::parameters::number_of_iterations(nb_iter)
                   .protect_constraints(protect)
                   .edge_is_constrained_map(selection_item->constrained_edges_pmap())
                   .relax_constraints(smooth_features)
@@ -451,7 +522,7 @@ public Q_SLOTS:
                 CGAL::Polygon_mesh_processing::isotropic_remeshing(selection_item->selected_facets
                   , target_length
                   , *selection_item->polyhedron()
-                  , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter)
+                  , CGAL::parameters::number_of_iterations(nb_iter)
                   .protect_constraints(protect)
                   .edge_is_constrained_map(selection_item->constrained_edges_pmap())
                   .relax_constraints(smooth_features)
@@ -459,41 +530,7 @@ public Q_SLOTS:
                   .vertex_is_constrained_map(selection_item->constrained_vertices_pmap()));
             }
         }
-        
-        SMesh mesh_ = *selection_item->polyhedron();
-        std::vector<bool> are_edges_removed;
-        are_edges_removed.resize(mesh_.number_of_edges()+mesh_.number_of_removed_edges());
-        std::vector<bool> are_edges_constrained;
-        are_edges_constrained.resize(are_edges_removed.size());
-        for(std::size_t i=0; i< are_edges_removed.size(); ++i)
-        {
-          are_edges_removed[i] = mesh_.is_removed(SMesh::Edge_index(static_cast<int>(i)));
-          if(!are_edges_removed[i])
-            are_edges_constrained[i] = get(selection_item->constrained_edges_pmap(), SMesh::Edge_index(static_cast<int>(i)));
-        }
-        
-        
-        int i0, i1,
-            nE(mesh_.number_of_edges()+mesh_.number_of_removed_edges());
-        
-        //get constrained values in order.
-        if (nE > 0)
-        {
-          i0=0;  i1=nE-1;
-          while (1)
-          {
-            // find first removed and last un-removed
-            while (!are_edges_removed[i0] && i0 < i1) ++i0;
-            while ( are_edges_removed[i1] && i0 < i1) --i1;
-            if (i0 >= i1) break;
-            
-            // swap
-            std::swap(are_edges_constrained[i0], are_edges_constrained[i1]);
-            std::swap(are_edges_removed[i0], are_edges_removed[i1]);
-          }
-          // remember new size
-          nE = are_edges_removed[i0] ? i0 : i0+1;
-        }
+
         selection_item->polyhedron_item()->setColor(
               selection_item->polyhedron_item()->color());
         if(fpmap_valid)
@@ -505,19 +542,11 @@ public Q_SLOTS:
         {
           selection_item->polyhedron_item()->setItemIsMulticolor(false);
         }
-        
-        selection_item->polyhedron_item()->polyhedron()->collect_garbage();
-        //fix constrained_edges_map
-        for(int i=0; i< nE; ++i)
-        {
-          Scene_polyhedron_selection_item::Is_constrained_map<Scene_polyhedron_selection_item::Selection_set_edge>
-              pmap = selection_item->constrained_edges_pmap();
-          put(pmap, SMesh::Edge_index(i), are_edges_constrained[i]);
-        }
-        
-        selection_item->poly_item_changed();
-        selection_item->clear<face_descriptor>();
-        selection_item->changed_with_poly_item();
+        selection_item->setKeepSelectionValid(Scene_polyhedron_selection_item::Edge);
+        selection_item->polyhedron_item()->invalidateOpenGLBuffers();
+        Q_EMIT selection_item->polyhedron_item()->itemChanged();
+        selection_item->invalidateOpenGLBuffers();
+        selection_item->setKeepSelectionValid(Scene_polyhedron_selection_item::None);
       }
       else if (poly_item)
       {
@@ -539,7 +568,7 @@ public Q_SLOTS:
                 edges_to_split
                 , target_length
                 , pmesh
-                , PMP::parameters::geom_traits(EPICK())
+                , CGAL::parameters::geom_traits(EPICK())
                 . edge_is_constrained_map(eif)
                 . face_patch_map(fpmap));
             else
@@ -547,7 +576,7 @@ public Q_SLOTS:
                 edges_to_split
                 , target_length
                 , pmesh
-                , PMP::parameters::geom_traits(EPICK())
+                , CGAL::parameters::geom_traits(EPICK())
                 . edge_is_constrained_map(eif));
           }
           else
@@ -563,7 +592,6 @@ public Q_SLOTS:
           if (preserve_duplicates)
           {
             detect_and_split_duplicates(poly_items, edges_to_protect_map, target_length);
-           
           }
           Scene_polyhedron_selection_item::Is_constrained_map<Edge_set> ecm(&edges_to_protect);
           for(edge_descriptor e : edges(pmesh))
@@ -577,7 +605,7 @@ public Q_SLOTS:
                pmesh,
                ecm,
                get(CGAL::vertex_point, pmesh),
-               CGAL::Static_property_map<face_descriptor, std::size_t>(1),
+               CGAL::Constant_property_map<face_descriptor, std::size_t>(1),
                4. / 3. * target_length))
           {
             QApplication::restoreOverrideCursor();
@@ -587,12 +615,31 @@ public Q_SLOTS:
                                     " Aborting."));
             return;
           }
+
+          if (!CGAL::is_triangle_mesh(pmesh))
+          {
+            QApplication::restoreOverrideCursor();
+            if (QMessageBox::Ok ==
+                QMessageBox::question(mw, tr("Error - Triangulate Faces?"),
+                  tr("The input mesh is not a triangulated surface mesh.\n"
+                     "Do you wish to triangulate faces first, or cancel remeshing ?"),
+                  (QMessageBox::Ok | QMessageBox::Cancel), QMessageBox::Ok))
+            {
+              QApplication::setOverrideCursor(Qt::WaitCursor);
+              CGAL::Polygon_mesh_processing::triangulate_faces(pmesh);
+            }
+            else
+            {
+              return;
+            }
+          }
+
           if (fpmap_valid)
             CGAL::Polygon_mesh_processing::isotropic_remeshing(
                  faces(*poly_item->polyhedron())
                , target_length
                , *poly_item->polyhedron()
-               , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter)
+               , CGAL::parameters::number_of_iterations(nb_iter)
                .protect_constraints(protect)
                .number_of_relaxation_steps(nb_smooth)
                .edge_is_constrained_map(ecm)
@@ -603,15 +650,20 @@ public Q_SLOTS:
                  faces(*poly_item->polyhedron())
                , target_length
                , *poly_item->polyhedron()
-               , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter)
+               , CGAL::parameters::number_of_iterations(nb_iter)
                .protect_constraints(protect)
                .number_of_relaxation_steps(nb_smooth)
                .edge_is_constrained_map(ecm)
                .relax_constraints(smooth_features));
+
+          //recollect sharp edges
+          for(edge_descriptor e : edges(pmesh))
+            eif[e] = false;
+          for(edge_descriptor e : edges_to_protect)
+            eif[e] = true;
         }
         if (fpmap_valid)
         {
-          PMP::connected_components(pmesh, fpmap, PMP::parameters::edge_is_constrained_map(eif));
           poly_item->setItemIsMulticolor(true);
           poly_item->show_feature_edges(true);
         }
@@ -626,10 +678,9 @@ public Q_SLOTS:
         std::cout << "Can't remesh that type of thing" << std::endl;
       }
       std::cout << "ok (" << time.elapsed() << " ms)" << std::endl;
-
-      // default cursor
-      QApplication::restoreOverrideCursor();
     }
+    // default cursor
+    QApplication::restoreOverrideCursor();
   }
 
   void isotropic_remeshing_of_several_polyhedra()
@@ -642,12 +693,12 @@ public Q_SLOTS:
     bool smooth_features = true;
 
     std::vector<Scene_facegraph_item*> selection;
-    BOOST_FOREACH(int index, scene->selectionIndices())
+    for(int index : scene->selectionIndices())
     {
       Scene_facegraph_item* poly_item =
         qobject_cast<Scene_facegraph_item*>(scene->item(index));
 
-      if (poly_item == NULL)
+      if (poly_item == nullptr)
       {
         std::cout << scene->item(index)->name().data()
           << " is not a FaceGraph, remeshing skipped\n";
@@ -660,7 +711,7 @@ public Q_SLOTS:
         if (target_length == 0.)//parameters have not been set yet
         {
         QDialog dialog(mw);
-        Ui::Isotropic_remeshing_dialog ui = remeshing_dialog(&dialog, poly_item);
+        initialize_remeshing_dialog(&dialog, poly_item);
         ui.objectName->setText(QString::number(scene->selectionIndices().size())
           .append(QString(" items to be remeshed")));
         int i = dialog.exec();
@@ -686,12 +737,36 @@ public Q_SLOTS:
       return;
     }
 
+
+    //check non-triangulated surfaces
+    for (Scene_facegraph_item* poly_item : selection)
+    {
+      if (!CGAL::is_triangle_mesh(*poly_item->polyhedron()))
+      {
+        if (QMessageBox::Ok == QMessageBox::question(mw,
+              tr("Error - Triangulate Faces?"),
+              tr("The input mesh ").append(poly_item->name())
+               .append(tr(" is not a triangulated surface mesh.\n"
+                "Do you wish to triangulate faces first, or cancel remeshing ?")),
+              (QMessageBox::Ok | QMessageBox::Cancel),
+              QMessageBox::Ok))
+        {
+          QApplication::setOverrideCursor(Qt::WaitCursor);
+          CGAL::Polygon_mesh_processing::triangulate_faces(*poly_item->polyhedron());
+          QApplication::restoreOverrideCursor();
+        }
+        else
+        {
+          QApplication::restoreOverrideCursor();
+          return;
+        }
+      }
+    }
+
     // wait cursor
     QApplication::setOverrideCursor(Qt::WaitCursor);
     int total_time = 0;
 
-
-    //     typedef boost::graph_traits<FaceGraph>::edge_descriptor edge_descriptor;
     std::map<FaceGraph*,Edge_set > edges_to_protect;
 
     if(preserve_duplicates)
@@ -712,7 +787,7 @@ public Q_SLOTS:
 
     Remesh_polyhedron_item remesher(edges_only,
       target_length, nb_iter, protect, smooth_features);
-    BOOST_FOREACH(Scene_facegraph_item* poly_item, selection)
+    for(Scene_facegraph_item* poly_item : selection)
     {
       QElapsedTimer time;
       time.start();
@@ -727,7 +802,7 @@ public Q_SLOTS:
     std::cout << "Remeshing of all selected items done in "
       << total_time << " ms" << std::endl;
 
-    BOOST_FOREACH(Scene_facegraph_item* poly_item, selection)
+    for(Scene_facegraph_item* poly_item : selection)
     {
       //destroys the patch_id_map for the Surface_mesh_item to avoid assertions.
       poly_item->resetColors();
@@ -768,7 +843,7 @@ private:
           , *poly_item->polyhedron()
           , std::back_inserter(border));
         std::vector<edge_descriptor> border_edges;
-        BOOST_FOREACH(halfedge_descriptor h, border)
+        for(halfedge_descriptor h : border)
           border_edges.push_back(edge(h, *poly_item->polyhedron()));
 
         CGAL::Polygon_mesh_processing::split_long_edges(
@@ -785,7 +860,7 @@ private:
             faces(*poly_item->polyhedron())
           , target_length_
           , *poly_item->polyhedron()
-          , CGAL::Polygon_mesh_processing::parameters::number_of_iterations(nb_iter_)
+          , CGAL::parameters::number_of_iterations(nb_iter_)
           .protect_constraints(protect_)
           .edge_is_constrained_map(ecm)
           .face_patch_map(get(CGAL::face_patch_id_t<int>(), *poly_item->polyhedron()))
@@ -863,36 +938,79 @@ private:
   };
 #endif
 
-  Ui::Isotropic_remeshing_dialog
-  remeshing_dialog(QDialog* dialog,
-                   Scene_facegraph_item* poly_item,
-                   Scene_polyhedron_selection_item* selection_item = NULL)
+public Q_SLOTS:
+  void update_after_protect_checkbox_click()
+  {
+    if(ui.protect_checkbox->isChecked())
+    {
+      ui.smooth1D_label->setEnabled(false);
+      ui.smooth1D_checkbox->setEnabled(false);
+      ui.smooth1D_checkbox->setChecked(false);
+    }
+    else
+    {
+      ui.smooth1D_label->setEnabled(true);
+      ui.smooth1D_checkbox->setEnabled(true);
+    }
+  }
+
+  void update_after_splitEdgesOnly_click()
+  {
+    if(ui.splitEdgesOnly_checkbox->isChecked())
+    {
+      ui.nbIterations_label->setEnabled(false);
+      ui.nbIterations_spinbox->setEnabled(false);
+      ui.nbSmoothing_label->setEnabled(false);
+      ui.nbSmoothing_spinbox->setEnabled(false);
+
+      ui.protect_label->setEnabled(false);
+      ui.protect_checkbox->setEnabled(false);
+      ui.protect_checkbox->setChecked(false);
+
+      ui.smooth1D_label->setEnabled(false);
+      ui.smooth1D_checkbox->setEnabled(false);
+      ui.smooth1D_checkbox->setChecked(false);
+    }
+    else
+    {
+      ui.nbIterations_label->setEnabled(true);
+      ui.nbIterations_spinbox->setEnabled(true);
+      ui.nbSmoothing_label->setEnabled(true);
+      ui.nbSmoothing_spinbox->setEnabled(true);
+
+      ui.protect_label->setEnabled(true);
+      ui.protect_checkbox->setEnabled(true);
+
+      ui.smooth1D_label->setEnabled(true);
+      ui.smooth1D_checkbox->setEnabled(true);
+    }
+  }
+
+public:
+  void
+  initialize_remeshing_dialog(QDialog* dialog,
+                              Scene_facegraph_item* poly_item,
+                              Scene_polyhedron_selection_item* selection_item = nullptr)
   {
-    Ui::Isotropic_remeshing_dialog ui;
     ui.setupUi(dialog);
     connect(ui.buttonBox, SIGNAL(accepted()), dialog, SLOT(accept()));
     connect(ui.buttonBox, SIGNAL(rejected()), dialog, SLOT(reject()));
 
     //connect checkbox to spinbox
-    connect(ui.splitEdgesOnly_checkbox, SIGNAL(toggled(bool)),
-            ui.nbIterations_spinbox, SLOT(setDisabled(bool)));
-    connect(ui.splitEdgesOnly_checkbox, SIGNAL(toggled(bool)),
-            ui.protect_checkbox, SLOT(setDisabled(bool)));
-    connect(ui.protect_checkbox, SIGNAL(toggled(bool)),
-            ui.smooth1D_checkbox, SLOT(setDisabled(bool)));
-    connect(ui.splitEdgesOnly_checkbox, SIGNAL(toggled(bool)),
-            ui.smooth1D_checkbox, SLOT(setDisabled(bool)));
     connect(ui.preserveDuplicates_checkbox, SIGNAL(toggled(bool)),
             ui.protect_checkbox, SLOT(setChecked(bool)));
     connect(ui.preserveDuplicates_checkbox, SIGNAL(toggled(bool)),
             ui.protect_checkbox, SLOT(setDisabled(bool)));
 
+    connect(ui.protect_checkbox, SIGNAL(clicked(bool)), this, SLOT(update_after_protect_checkbox_click()));
+    connect(ui.splitEdgesOnly_checkbox, SIGNAL(clicked(bool)), this, SLOT(update_after_splitEdgesOnly_click()));
+
     //Set default parameters
-    Scene_interface::Bbox bbox = poly_item != NULL ? poly_item->bbox()
-      : (selection_item != NULL ? selection_item->bbox()
+    Scene_interface::Bbox bbox = poly_item != nullptr ? poly_item->bbox()
+      : (selection_item != nullptr ? selection_item->bbox()
         : scene->bbox());
-    ui.objectName->setText(poly_item != NULL ? poly_item->name()
-      : (selection_item != NULL ? selection_item->name()
+    ui.objectName->setText(poly_item != nullptr ? poly_item->name()
+      : (selection_item != nullptr ? selection_item->name()
         : QString("Remeshing parameters")));
 
     ui.objectNameSize->setText(
@@ -904,13 +1022,8 @@ private:
     double diago_length = CGAL::sqrt((bbox.xmax()-bbox.xmin())*(bbox.xmax()-bbox.xmin())
                                    + (bbox.ymax()-bbox.ymin())*(bbox.ymax()-bbox.ymin())
                                    + (bbox.zmax()-bbox.zmin())*(bbox.zmax()-bbox.zmin()));
-    double log = std::log10(diago_length);
-    unsigned int nb_decimals = (log > 0) ? 5 : (std::ceil(-log)+3);
 
-    ui.edgeLength_dspinbox->setDecimals(nb_decimals);
-    ui.edgeLength_dspinbox->setSingleStep(1e-3);
-    ui.edgeLength_dspinbox->setRange(1e-6 * diago_length, //min
-                                     2.   * diago_length);//max
+
     ui.edgeLength_dspinbox->setValue(0.05 * diago_length);
 
     std::ostringstream oss;
@@ -926,20 +1039,18 @@ private:
     ui.protect_checkbox->setChecked(false);
     ui.smooth1D_checkbox->setChecked(true);
 
-    if (NULL != selection_item)
+    if (nullptr != selection_item)
     {
       //do not preserve duplicates in selection mode
       ui.preserveDuplicates_checkbox->setDisabled(true);
       ui.preserveDuplicates_checkbox->setChecked(false);
     }
-
-    return ui;
   }
 
 
 private:
   QAction* actionIsotropicRemeshing_;
-
+  Ui::Isotropic_remeshing_dialog ui;
 }; // end Polyhedron_demo_isotropic_remeshing_plugin
 
 #include "Isotropic_remeshing_plugin.moc"
