@@ -2,12 +2,16 @@
 
 #include <fstream>
 #include <vector>
+#include <deque>
+#include <boost/config.hpp>
+#include <boost/version.hpp>
 
 // CGAL headers
 #include <CGAL/Bbox_2.h>
 #include <CGAL/assertions_behaviour.h>
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
-#include <CGAL/Timer.h> 
+#include <CGAL/Timer.h>
+#include <CGAL/IO/WKT.h>
 
 //#define CGAL_TESTING_POLYLINE_SIMPLIFICATION
 //#define CGAL_POLYLINE_SIMPLIFICATION_TRACE_LEVEL 15
@@ -16,7 +20,7 @@ bool lAppToLog = false ;
 void Polyline_simplification_2_external_trace( std::string m )
 {
   std::ofstream out("polysim_log.txt", ( lAppToLog ? std::ios::app | std::ios::ate : std::ios::trunc | std::ios::ate ) );
-  out << std::setprecision(19) << m << std::endl << std::flush ; 
+  out << std::setprecision(19) << m << std::endl << std::flush ;
   lAppToLog = true ;
 }
 
@@ -26,10 +30,10 @@ void error_handler ( char const* what, char const* expr, char const* file, int l
        << "Expr: " << expr << std::endl
        << "File: " << file << std::endl
        << "Line: " << line << std::endl;
-  if ( msg != 0)
+  if ( msg != nullptr)
       std::cerr << "Explanation:" << msg << std::endl;
-    
-  throw std::runtime_error("CGAL Error");  
+
+  throw std::runtime_error("CGAL Error");
 }
 
 
@@ -85,15 +89,15 @@ class MainWindow :
   public Ui::Polyline_simplification_2
 {
   Q_OBJECT
-  
-private:  
 
-  PCT                                                m_pct; 
+private:
+
+  PCT                                                m_pct;
   QGraphicsScene                                    mScene;
   CGAL::Qt::PolylineSimplificationGraphicsItem<PCT> * mGI;
   CGAL::Qt::GraphicsViewPolylineInput<K> *          mPI;
-  
-private:  
+
+private:
 
 public:
 
@@ -106,6 +110,7 @@ private:
 
   void loadPoly(QString);
   void loadOSM (QString);
+  void loadWKT (QString);
 
 protected Q_SLOTS:
 void open(QString);
@@ -115,9 +120,9 @@ public Q_SLOTS:
   void processInput(CGAL::Object o);
 
   void on_actionShowTriangulation_toggled(bool checked);
-  
+
   void on_actionInsertPolyline_toggled(bool checked);
-  
+
   void on_actionClear_triggered();
 
   void on_actionRecenter_triggered();
@@ -129,7 +134,7 @@ public Q_SLOTS:
   Mode getSimplificationMode() ;
 
   double getThreshold() ;
-  
+
 Q_SIGNALS:
   void changed();
 };
@@ -140,14 +145,14 @@ MainWindow::MainWindow()
 {
   CGAL::set_error_handler  (error_handler);
   CGAL::set_warning_handler(error_handler);
-  
+
   setupUi(this);
 
   setAcceptDrops(true);
 
   // Add a GraphicItem for the PS triangulation
   mGI = new CGAL::Qt::PolylineSimplificationGraphicsItem<PCT>(&m_pct);
-    
+
   QObject::connect(this, SIGNAL(changed()), mGI, SLOT(modelChanged()));
 
   mGI->setVerticesPen(QPen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
@@ -155,18 +160,18 @@ MainWindow::MainWindow()
   mGI->setZValue(-1);
   mGI->setVisibleEdges(false);
   mGI->setVisibleConstraints(true);
-  
+
   mScene.addItem(mGI);
 
   // Setup input handlers. They get events before the mScene gets them
-  // and the input they generate is passed to the triangulation with 
-  // the signal/slot mechanism    
+  // and the input they generate is passed to the triangulation with
+  // the signal/slot mechanism
   mPI = new CGAL::Qt::GraphicsViewPolylineInput<K>(this, &mScene, 0, true); // inputs polylines which are not closed
   this->on_actionInsertPolyline_toggled(true);
   QObject::connect(mPI, SIGNAL(generate(CGAL::Object)), this, SLOT(processInput(CGAL::Object)));
-    
 
-  // 
+
+  //
   // Manual handling of actions
   //
   QObject::connect(this->actionQuit, SIGNAL(triggered()), this, SLOT(close()));
@@ -176,7 +181,7 @@ MainWindow::MainWindow()
   //  ag->addAction(this->actionInsertPolyline);
 
   this->actionShowTriangulation->setChecked(false);
-  
+
   //
   // Setup the mScene and the view
   //
@@ -187,7 +192,7 @@ MainWindow::MainWindow()
 
   // Turn the vertical axis upside down
   this->graphicsView->scale(1, -1);
-                                                      
+
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
   this->addNavigation(this->graphicsView);
@@ -196,24 +201,23 @@ MainWindow::MainWindow()
   this->setupOptionsMenu();
   this->addAboutDemo(":/cgal/help/about_Polyline_simplification_2.html");
   this->addAboutCGAL();
+#if QT_SVG_LIB
   this->setupExportSVG(action_Export_SVG, graphicsView);
+#endif
 
   this->addRecentFiles(this->menuFile, this->actionQuit);
   connect(this, SIGNAL(openRecentFile(QString)), this, SLOT(open(QString)));
-	  
-//  QObject::connect(thresholdSlider, SIGNAL(valueChanged(int)), this, SLOT(set_Threshold(int)));
-	  
 }
 
 
-void 
+void
 MainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
   if (event->mimeData()->hasFormat("text/uri-list"))
     event->acceptProposedAction();
 }
 
-void 
+void
 MainWindow::dropEvent(QDropEvent *event)
 {
   QString filename = event->mimeData()->urls().at(0).path();
@@ -227,7 +231,7 @@ MainWindow::processInput(CGAL::Object o)
   std::list<Point_2> points;
   if(CGAL::assign(points, o))
   {
-    if(points.size() >= 2) 
+    if(points.size() >= 2)
     {
       m_pct.insert_constraint(points.begin(), points.end());
 #if 0
@@ -239,16 +243,16 @@ MainWindow::processInput(CGAL::Object o)
       }
 #endif
       Q_EMIT( changed());
-    }  
+    }
   }
   Q_EMIT( changed());
 }
 
 
-/* 
+/*
  *  Qt Automatic Connections
  *  https://doc.qt.io/qt-5/designer-using-a-ui-file.html#automatic-connections
- * 
+ *
  *  setupUi(this) generates connections to the slots named
  *  "on_<action_name>_<signal_name>"
  */
@@ -299,12 +303,12 @@ void MainWindow::on_actionSimplify_triggered()
   // wait cursor
   QApplication::setOverrideCursor(Qt::WaitCursor);
 
-  
+
   try
   {
     switch( getSimplificationMode() )
     {
-    case ABS_P   : simplify(m_pct, PS2::Squared_distance_cost(),  PS2::Stop_below_count_ratio_threshold(getThreshold())   ) ; 
+    case ABS_P   : simplify(m_pct, PS2::Squared_distance_cost(),  PS2::Stop_below_count_ratio_threshold(getThreshold())   ) ;
  break ;
     case ABS_E   : simplify(m_pct, PS2::Squared_distance_cost(), PS2::Stop_above_cost_threshold(getThreshold()) ) ; break ;
     case REL_P   : simplify(m_pct, PS2::Scaled_squared_distance_cost(),  PS2::Stop_below_count_ratio_threshold(getThreshold()) ) ; break ;
@@ -314,14 +318,14 @@ void MainWindow::on_actionSimplify_triggered()
 
       break ;
     }
-    
+
     statusBar()->showMessage(QString("Simplification done"));
-  }  
-  catch(...) 
-  {
-    statusBar()->showMessage(QString("Exception ocurred"));
   }
-  
+  catch(...)
+  {
+    statusBar()->showMessage(QString("Exception occurred"));
+  }
+
    // default cursor
   QApplication::restoreOverrideCursor();
   mGI->modelChanged();
@@ -329,18 +333,18 @@ void MainWindow::on_actionSimplify_triggered()
 }
 
 
-void 
+void
 MainWindow::open(QString fileName)
 {
   if(! fileName.isEmpty()){
-    if(fileName.endsWith(".poly")){
-      loadPoly(fileName);
-      this->addToRecentFiles(fileName);
-    }  
-    else if(fileName.endsWith(".osm")){
+    if(fileName.endsWith(".osm")){
       loadOSM(fileName);
       this->addToRecentFiles(fileName);
-    }  
+    }
+    else if(fileName.endsWith(".wkt")){
+      loadWKT(fileName);
+      this->addToRecentFiles(fileName);
+    }
   }
 }
 
@@ -348,10 +352,10 @@ void
 MainWindow::on_actionLoadConstraints_triggered()
 {
   QString fileName = QFileDialog::getOpenFileName(this,
-						                                      tr("Open Constraint File"),
-						                                      "../data",
-						                                      tr("Polylines (*.osm *.poly)")
-						                                      );
+                                                  tr("Open Constraint File"),
+                                                  "../data"
+                                                  ,tr("Polylines (*.osm *.wkt);;")
+                                                  );
   open(fileName);
 }
 
@@ -363,95 +367,52 @@ std::string trim_right ( std::string str )
     if ( pos != std::string::npos )
       return str.substr(0,pos+1);
   }
-  
-  return std::string("") ;  
+
+  return std::string("") ;
 }
 
-enum { POLYGON, POLYLINE, POINT_2 } ;
-
-void MainWindow::loadPoly(QString fileName)
+void MainWindow::loadWKT(QString fileName)
 {
+    typedef std::vector<Point_2> MultiPoint;
+
+  typedef std::vector<Point_2> LineString;
+  typedef std::deque<LineString> MultiLineString;
+
+  typedef CGAL::Polygon_2<K> Polygon_2;
+  typedef CGAL::Polygon_with_holes_2<K> Polygon_with_holes_2;
+  typedef std::deque<Polygon_with_holes_2> MultiPolygon;
+
+  std::ifstream ifs(qPrintable(fileName));
+  MultiPoint points;
+  MultiLineString polylines;
+  MultiPolygon polygons;
+  CGAL::IO::read_WKT(ifs, points,polylines,polygons);
+
   m_pct.clear();
   mGI->modelChanged();
-  
-  std::ifstream ifs(qPrintable(fileName));
-  
-  std::string line ;
-  
-  std::vector<Point_2> poly ;
-  
-  bool lIsClosed = true ;
-  
-  while ( std::getline(ifs,line) )
-  {
-    line = trim_right(line);
-    
-    if ( line.size() > 0 )
-    {
-      int lCode = POINT_2 ;
-      
-      if ( line == "POLYGON" )    
-      {
-        lCode = POLYGON ; 
-        lIsClosed = true ; 
-      }
-      else if ( line == "POLYLINE" )
-      {
-        lCode = POLYLINE ;
-        lIsClosed = false ; 
-      }
-      
-      if ( lCode == POINT_2 )
-      {
-        double x,y ;
-        
-        std::istringstream ss(line);
-        
-        ss >> x >> y ;
-        
-        poly.push_back( Point_2(x,y) );
-      }
-      else
-      {
-        if ( poly.size() > 0 )  
-        {
-          if ( lIsClosed )
-          {
-            if(poly.back() != poly.front()){
-              poly.push_back(poly.front());
-            }
-            m_pct.insert_constraint(poly.begin(), poly.end() ) ;
-          }
-          else
-          {
-            m_pct.insert_constraint(poly.begin(), poly.end() ) ;
-          }
-        }
-        poly.clear();
-      }
+
+  if(! points.empty()){
+    std::cout << "Ignore " << points.size() << " isolated points" << std::endl;
+  }
+  for(LineString poly : polylines){
+    if ( poly.size() > 2 ){
+      m_pct.insert_constraint(poly.begin(), poly.end());
     }
   }
-  
-  if ( poly.size() > 0 )  
-  {
-    if ( lIsClosed )
-    {
-      if(poly.back() != poly.front()){
-        poly.push_back(poly.front());
-      }
-      m_pct.insert_constraint(poly.begin(), poly.end() ) ;
+ for(const Polygon_with_holes_2& poly : polygons){
+   m_pct.insert_constraint(poly.outer_boundary().vertices_begin(), poly.outer_boundary().vertices_end());
+   for(Polygon_with_holes_2::Hole_const_iterator it = poly.holes_begin(); it != poly.holes_end(); ++it){
+     const Polygon_2& hole = *it;
+      m_pct.insert_constraint(hole);
     }
-    else
-    {
-      m_pct.insert_constraint(poly.begin(), poly.end() ) ;
-    }
-  }
-      
+ }
+
   Q_EMIT( changed());
-  
+
   actionRecenter->trigger();
 }
 
+
 void MainWindow::loadOSM(QString fileName)
 {
   m_pct.clear();
@@ -460,20 +421,20 @@ void MainWindow::loadOSM(QString fileName)
   try
   {
     std::ifstream ifs(qPrintable(fileName));
-    
+
     std::string line ;
-    
+
     std::vector<Point_2> poly ;
-    
+
     while ( std::getline(ifs,line) )
     {
       line = trim_right(line);
-      
+
       if ( line.size() > 0 )
       {
         if ( line.find(':') != std::string::npos )
         {
-          if ( poly.size() > 0 )  
+          if ( poly.size() > 0 )
           {
             if ( poly.front() == poly.back() && poly.size() >= 4 )
             {
@@ -491,21 +452,21 @@ void MainWindow::loadOSM(QString fileName)
         else
         {
           double x,y ;
-          
+
           std::string::size_type pos = line.find(',');
           if ( pos != std::string::npos )
             line[pos]= ' ' ;
-          
+
           std::istringstream ss(line);
-          
+
           ss >> x >> y ;
-          
+
           poly.push_back( Point_2(x,y) );
         }
       }
     }
-    
-    if ( poly.size() > 0 )  
+
+    if ( poly.size() > 0 )
     {
       if ( poly.front() == poly.back() )
       {
@@ -516,14 +477,14 @@ void MainWindow::loadOSM(QString fileName)
         m_pct.insert_constraint(poly.begin(), poly.end() ) ;
       }
     }
-  }  
-  catch(...) 
-  {
-    statusBar()->showMessage(QString("Exception ocurred"));
   }
-  
+  catch(...)
+  {
+    statusBar()->showMessage(QString("Exception occurred"));
+  }
+
   Q_EMIT( changed());
-  
+
   actionRecenter->trigger();
 }
 
@@ -531,7 +492,7 @@ void
 MainWindow::on_actionRecenter_triggered()
 {
   this->graphicsView->setSceneRect(mGI->boundingRect());
-  this->graphicsView->fitInView(mGI->boundingRect(), Qt::KeepAspectRatio);  
+  this->graphicsView->fitInView(mGI->boundingRect(), Qt::KeepAspectRatio);
 }
 
 #include "Polyline_simplification_2.moc"
@@ -546,7 +507,7 @@ int main(int argc, char **argv)
 
   // Import resources from libCGALQt5.
   CGAL_QT_INIT_RESOURCES;
-	
+
   MainWindow mainWindow;
   mainWindow.show();
   return app.exec();
